@@ -1,9 +1,9 @@
 ##
 ##  Implementation of the 2004 Pham et al. paper
 ##
-##  Created by Daniel Rodríguez Pérez on 6/9/2014.
+##  Created by Daniel Rodriguez Perez on 6/9/2014.
 ##
-##  Copyright (c) 2014 Daniel Rodríguez Pérez.
+##  Copyright (c) 2014 Daniel Rodriguez Perez.
 ##
 ##  This program is free software: you can redistribute it and/or modify
 ##  it under the terms of the GNU General Public License as published by
@@ -25,14 +25,20 @@
 #' 
 #' @param x numeric matrix of data, or an object that can be coerced to such a
 #'        matrix.
+#' @param fun_cluster function to cluster by (e.g. \code{kmeans}). The first 
+#'        parameter of the function must a numeric matrix and the second the
+#'        number of clusters. The function must return an object with a named
+#'        attribute \code{withinss} which is a numeric vector with the within.
 #' @param max_centers maximum number of clusters for evaluation.
 #' @param k_threshold maximum value of \eqn{f(K)} from which can not be
 #'        considered the existence of more than one cluster in the data set.
 #'        The default value is 0.85.
-#' @param nstart the number of random sets that should be chosen in the kmeans
-#'        method.
 #' @param progressBar show a progress bar.
 #' @param trace display a trace of the progress.
+#' @param parallel If set to true, use parallel \code{foreach} to execute the
+#'        function that implements the kmeans algorithm. Must register parallel
+#'        before hand, such as \code{doMC} or others. Selecting this option the
+#'        progress bar is disabled.
 #' @param ... arguments to be passed to the kmeans method.
 #' 
 #' @return an object with the \eqn{f(K)} results.
@@ -76,7 +82,7 @@
 #' dat <- matrix(c(rnorm(100, 2, .1), rnorm(100, 3, .1),
 #'                 rnorm(100, -2, .1), rnorm(100, -3, .1)), 200, 2)
 #'
-#' # Ejecute the method
+#' # Execute the method
 #' sol <- kselection(dat)
 #' 
 #' # Get the results
@@ -86,7 +92,16 @@
 #' # Plot the results
 #' plot(sol)
 #' 
-#' @author Daniel Rodriguez Perez
+#' \dontrun{
+#' # Parallel
+#' require(doMC)
+#' registerDoMC(cores = 4)
+#' 
+#' system.time(kselection(dat, max_centers = 50 , nstart = 25))
+#' system.time(kselection(dat, max_centers = 50 , nstart = 25, parallel = TRUE))
+#' }
+#' 
+#' @author Daniel Rodriguez
 #' 
 #' @references
 #' D T Pham, S S Dimov, and C D Nguyen, "Selection of k in k-means clustering",
@@ -99,14 +114,33 @@
 #' @rdname kselection
 #' @export kselection
 kselection <- function(x,
+                       fun_cluster = stats::kmeans,
                        max_centers = 15,
                        k_threshold = 0.85,
-                       nstart      = 25,
                        progressBar = FALSE,
-                       trace       = FALSE, ...) {
+                       trace       = FALSE,
+                       parallel    = FALSE, ...) {
+  if (!is.function(fun_cluster))
+    stop("'fun_cluster' must be a function.")
+  
   if (max_centers < 2)
     stop("'max_centers' must be greater than 2")
-    
+  
+  if (!is.logical(progressBar)) {
+    progressBar <- FALSE
+    warning("'progressBar' must be a logical")
+  }
+  
+  if (!is.logical(trace)) {
+    trace <- FALSE
+    warning("'trace' must be a logical")
+  }
+  
+  if (!is.logical(parallel)) {
+    parallel <- FALSE
+    warning("'parallel' must be a logical")  
+  }
+  
   x <- as.matrix(x)
   if (!is.numeric(x)) 
     stop('x must contain numerical data')
@@ -122,19 +156,31 @@ kselection <- function(x,
   f_k <- rep(1, max_centers)
   s_k <- rep(1, max_centers)
   a_k <- alpha_k(num_col, max_centers)
-  
-  if (progressBar) {
-    pb <- txtProgressBar(min   = 0,
-                         max   = max_centers,
-                         style = 3)
+    
+  if (parallel && requireNamespace('foreach', quietly = TRUE)) {
+    s_k <- foreach::"%dopar%"(
+      foreach::foreach(k = 1:max_centers, .combine = c), {
+        mod_info <- fun_cluster(x, k, ...)
+        s_k      <- sum(mod_info$withinss)
+      })
+  } else {
+    if (progressBar) {
+      pb <- txtProgressBar(min   = 0,
+                           max   = max_centers,
+                           style = 3)
+    }
+    
+    for (k in 1:max_centers) {
+      mod_info <- fun_cluster(x, k, ...)
+      s_k[k]   <- sum(mod_info$withinss)
+      
+      if (progressBar) {
+        setTxtProgressBar(pb, k)
+      }
+    }
   }
   
-  for (k in 1:max_centers) {
-    mod_info <- kmeans(x,
-                       centers = k,
-                       nstart  = nstart, ...)
-    s_k[k]   <- sum(mod_info$withinss)
-    
+  for (k in 1:max_centers) {    
     if (k == 1) {
       f_k[k] <- 1  
     } else {
@@ -148,34 +194,31 @@ kselection <- function(x,
     if (trace) {
       message('Number of clusters', k,' - f(k) = ', f_k[k])
     }
-    
-    if (progressBar) {
-      setTxtProgressBar(pb, k)
-    }
   }
   
   result <- list(k           = which_cluster(f_k, k_threshold),
                  f_k         = f_k,
                  max_centers = max_centers,
-                 k_threshold = k_threshold)
+                 k_threshold = k_threshold,
+                 fun_cluster = fun_cluster)
   class(result) <- 'Kselection'
   
   return(result)
 }
 
 #' Get the \code{k_threshold}
-#' 
+#'
 #' Get the maximum value of \eqn{f(K)} from which can not be considered the
 #' existence of more than one cluster.
-#' 
+#'
 #' @param obj the output of \code{kselection} function.
-#' 
+#'
 #' @return the \code{k_threshold} value.
 #' 
-#' @author Daniel Rodriguez Perez
+#' @author Daniel Rodriguez
 #' 
 #' @seealso \code{\link{set_k_threshold}}
-#' 
+#'
 #' @rdname get_k_threshold
 #' @export get_k_threshold
 get_k_threshold <- function(obj) {
@@ -195,20 +238,20 @@ get_k_threshold.Kselection <- function(obj) {
 }
 
 #' Set the \code{k_threshold}
-#' 
+#'
 #' Set the maximum value of \eqn{f(K)} from which can not be considered the
 #' existence of more than one cluster.
-#' 
+#'
 #' @param obj the output of \code{kselection} function.
 #' @param k_threshold maximum value of \eqn{f(K)} from which can not be
 #'        considered the existence of more than one cluster in the data set.
-#'        
+#'
 #' @return the output of kselection function with new \code{k_threshold}.
 #' 
-#' @author Daniel Rodriguez Perez
+#' @author Daniel Rodriguez
 #' 
 #' @seealso \code{\link{get_k_threshold}}
-#' 
+#'
 #' @rdname set_k_threshold
 #' @export set_k_threshold
 set_k_threshold <- function(obj, k_threshold) {
@@ -220,36 +263,36 @@ set_k_threshold <- function(obj, k_threshold) {
 set_k_threshold.Kselection <- function(obj, k_threshold) {
   if (length(k_threshold) != 1L)
     stop('k_threshold must be scalar')
-  if (!is.numeric(k_threshold)) 
+  if (!is.numeric(k_threshold))
     stop('k_threshold must be numeric')
   if (k_threshold <= 0)
     stop('k_threshold must be numeric bigger than 0')
-  
+
   obj$k_threshold <- k_threshold
   obj
 }
 
 #' Get the \eqn{f(K)} vector
-#' 
+#'
 #' Get the \eqn{f(K)} vector.
-#' 
+#'
 #' @param obj the output of \code{kselection} function.
-#' 
+#'
 #' @return the vector of \eqn{f(K)} function.
-#' 
+#'
 #' @examples
 #' # Create a data set with two clusters
 #' dat <- matrix(c(rnorm(100, 2, .1), rnorm(100, 3, .1),
 #'                 rnorm(100, -2, .1), rnorm(100, -3, .1)), 200, 2)
-#'               
+#'
 #' # Get the f(k) vector
 #' sol <- kselection(dat)
 #' f_k <- get_f_k(sol)
 #' 
-#' @author Daniel Rodriguez Perez
+#' @author Daniel Rodriguez
 #' 
 #' @seealso \code{\link{num_clusters}}, \code{\link{num_clusters_all}}
-#' 
+#'
 #' @rdname get_f_k
 #' @export get_f_k
 get_f_k <- function(obj) {
@@ -269,26 +312,26 @@ get_f_k.Kselection <- function(obj) {
 }
 
 #' Get the number of clusters.
-#' 
+#'
 #' The optimal number of clusters proposed by the method.
-#' 
+#'
 #' @param obj the output of kselection function.
-#' 
+#'
 #' @return the number of clusters proposed.
-#' 
+#'
 #' @examples
 #' # Create a data set with two clusters
 #' dat <- matrix(c(rnorm(100, 2, .1), rnorm(100, 3, .1),
 #'                 rnorm(100, -2, .1), rnorm(100, -3, .1)), 200, 2)
-#'               
+#'
 #' # Get the optimal number of clustes
 #' sol <- kselection(dat)
 #' k   <- num_clusters(sol)
 #' 
-#' @author Daniel Rodriguez Perez
+#' @author Daniel Rodriguez
 #' 
 #' @seealso \code{\link{num_clusters_all}}, \code{\link{get_f_k}}
-#' 
+#'
 #' @rdname num_clusters
 #' @export num_clusters
 num_clusters <- function(obj) {
@@ -308,24 +351,24 @@ num_clusters.Kselection <- function(obj) {
 }
 
 #' Get all recommended numbers of clusters
-#' 
+#'
 #' The number of cluster which could be recommender according the method
 #' threshold.
-#' 
+#'
 #' @param obj the output of \code{kselection} function.
-#' 
+#'
 #' @return an array of number of clusters that could be recommended.
-#' 
+#'
 #' @examples
 #' # Create a data set with two clusters
 #' dat <- matrix(c(rnorm(100, 2, .1), rnorm(100, 3, .1),
 #'                 rnorm(100, -2, .1), rnorm(100, -3, .1)), 200, 2)
-#'               
+#'
 #' # Get the optimal number of clustes
 #' sol <- kselection(dat)
 #' k   <- num_clusters(sol)
 #' 
-#' @author Daniel Rodriguez Perez
+#' @author Daniel Rodriguez
 #' 
 #' @seealso \code{\link{num_clusters}}, \code{\link{get_f_k}}
 #' 
@@ -352,7 +395,7 @@ num_clusters_all.Kselection <- function(obj) {
 plot.Kselection <- function(x, ...) {
   max_y   <- 1.1 * max(x$f_k)
   valid_k <- num_clusters_all(x)
-  
+
   plot(x$f_k,
        main = to.string(x),
        type = 'b',
@@ -393,15 +436,15 @@ print.Kselection <- function(x, ...) {
 #
 # @return an array with the weights.
 # 
-# @author Daniel Rodriguez Perez
+# @author Daniel Rodriguez
 alpha_k <- function(n_d, k) {
   result <- 1 - 3/(4 * n_d)
   result <- rep(result, k)
   
-  for (k in 3:k) {
-    result[k] <- result[k - 1] + (1 - result[k - 1]) / 6
+  for (i in 3:k) {
+    result[i] <- result[i - 1] + (1 - result[i - 1]) / 6
   }
-  
+
   return(result)
 }
 
@@ -414,7 +457,7 @@ alpha_k <- function(n_d, k) {
 #
 # @return the number of clusters.
 # 
-# @author Daniel Rodriguez Perez
+# @author Daniel Rodriguez
 which_cluster <- function(f_k, k_threshold) {
   k <- which(f_k == min(f_k) & f_k < k_threshold)
   if (length(k) == 0)
@@ -428,10 +471,10 @@ which_cluster <- function(f_k, k_threshold) {
 # Generate a string with the recomender number of k.
 #
 # @param obj the output of kselection function.
-# 
+#
 # @return an string with the recomendation.
 #
-# @author Daniel Rodriguez Perez
+# @author Daniel Rodriguez
 to.string <- function(obj) {
   if (num_clusters(obj) == 1)
     paste('f(k) finds', num_clusters(obj), 'cluster')
